#pragma rtGlobals=1		// Use modern global access method.
#pragma IgorVersion = 6.1	//Runs only with version 6.1(B05) or later


//populationwave combines many 1D waves to one 2d wave
//x of 1D becomes x of 2D
//use transpose XY to transpose


function PopulationWave(basename, outputname, number, [offset])
string basename, outputname
variable number, offset

variable counter, c2, xdim, timestep, xoffset, popcounter=0
string namestr, xunit, dataunit

if (ParamIsDefault(offset))
	offset = 0
endif

counter = offset
namestr = basename+num2str(counter)

if (waveexists($namestr))
	xdim = dimsize($namestr,0)
	timestep = dimdelta($namestr,0)
	xunit = WaveUnits($namestr,0)
	xoffset = DimOffset($namestr, 0)
	dataunit = WaveUnits($namestr, 1)
else
	print "No such wave: "+namestr
	return - 1
endif

make /o/n=(xdim,number) popwave = 0

do
	namestr = basename+num2str(counter)

	if (waveexists($namestr))
		duplicate /o $namestr, tempwave
		popwave[][popcounter]=tempwave[p]
	else
		print "No such wave: "+namestr
		redimension /n=(-1,popcounter) popwave
		counter = number * 10
	endif
	
	popcounter+=1
	counter+=1
while (counter < number+offset)

setscale /p x,xoffset,timestep,xunit popwave
setscale /p y,offset,1,"Trace #" popwave
setscale d -inf, inf, dataunit popwave
duplicate /o popwave, $outputname
killwaves /z tempwave, popwave

end

///////////////////////////////////////////

function TransposeXY(basename, outputname)
wave basename
string  outputname

variable xdim, ydim, xcounter, ycounter, xstep, ystep
string xlabel, ylabel
xdim = dimsize(basename, 0)
xstep = dimdelta(basename, 0)
xlabel = WaveUnits(basename,0)
ydim = dimsize(basename, 1)
ystep = dimdelta(basename, 1)
ylabel = WaveUnits(basename,1)

make /o/n=(ydim,xdim) transpwave = 0

//for(xcounter=0;xcounter<xdim;xcounter+=1)
//	for(ycounter=0;ycounter<ydim;ycounter+=1)
//		transpwave[ycounter][xcounter] = basename[xcounter][ycounter]
//	endfor
//endfor

MultiThread transpwave[][] = basename[q][p]	//faster

setscale /p y,0,xstep,xlabel transpwave
setscale /p x,0,ystep,ylabel transpwave
setscale d, -inf,inf,WaveUnits(basename, -1) transpwave

duplicate /o transpwave, $outputname
killwaves /z transpwave
end


//////////////////////////////////////////////////////////////////

function PopX2Traces(PopWave,basename)
wave PopWave
string basename

variable tracenumbers = dimsize(PopWave, 1)
variable xdim = dimsize(PopWave,0)
variable counter, xdelta, xoffset, nameoffset
string name, xlabel, ylabel

xlabel = Waveunits(PopWave, 0)
xdelta = DimDelta(PopWave, 0)
xoffset = DimOffset(PopWave, 0)
ylabel = WaveUnits(PopWave, 1)
variable ydelta = DimDelta(PopWave, 1)
variable yoffset = DimOffset(PopWave, 1)


for (counter=0;counter<tracenumbers;counter+=1)

	make /o/n=(xdim) PXT
	PXT[]=popwave[p][counter]
	name = basename+num2str(counter+yoffset)
	SetScale /P x, xoffset,xdelta,xlabel, PXT
	SetScale /P y, 0,1,ylabel, PXT
	duplicate /o PXT, $name
	
	
endfor

killwaves /z PXT
return tracenumbers
end

//////////////////////////////////////////////////////////////////

function PopY2Traces(PopWave,basename)
wave PopWave
string basename

variable tracenumbers = dimsize(PopWave, 0)
variable xdim = dimsize(PopWave,1)
variable counter
string name

string datalabel = WaveUnits(PoPWave, -1)
string ylabel = WaveUnits(PopWave, 1)
variable ydelta = DimDelta(PopWave, 1)
variable yoffset = DimOffset(PopWave, 1)
string xlabel = Waveunits(PopWave, 0)
variable xdelta = DimDelta(PopWave, 0)
variable xoffset = DimOffset(PopWave, 0)

for (counter=0;counter<tracenumbers;counter+=1)

	make /o/n=(xdim) PYT
	PYT[]=popwave[counter][q]
	SetScale /P x Yoffset,Ydelta,Ylabel, PYT
	Setscale /P y 0,1,Xlabel, PYT
	name = basename+num2str(counter+Xoffset)
	duplicate /o PYT, $name
	
	
endfor

killwaves /z PYT
return tracenumbers
end

////////////////////////////////////////////////////////////////////

Function TraceYFromGraph(graphNameStr, yWaveNameStr)
String graphNameStr, yWaveNameStr

String info=TraceInfo(graphNameStr, yWaveNameStr,0)
String range=StringByKey("YRANGE",Info)
String range1,range2
sscanf range,"[%[0-9,\*]][%[0-9,\*]]",range1,range2 // Scans (twice) for characters 0,1,2,3,4,5,6,7,8,9,comma,asterisk between brackets. 

variable output =  str2num(range2)

if(numtype(output)==2)
	return-1
else
	return output
EndIf

End

////////////////////////////////////////////////////////////////////


Function PopWaveFromWindow()

String TNL, trace, first
Variable NumTraces, ii, NumPoints, TraceY, HashPos

string TWName=WinName(0,1,1), Origin

TNL = tracenamelist(TWName,";",1)
NumTraces=ItemsInList(TNL)
First=StringFromList(0,TNL)


HashPos=StrSearch(First,"#",0)
	If(HashPos > 0)
		First=First[0,HashPos-1]		//remove "#n"
	Endif
	

NumPoints=Dimsize($First,0)		//Assume that NumPoints of all traces is the same
Make /o/n=(NumPoints,NumTraces) WinPop
Make /o/t/n=(NumTraces) WinPopOrigins
Setscale /p x,DimOffset($First,0),DimDelta($First,0),WaveUnits($First,0) WinPop
Setscale d,-inf,inf,WaveUnits($First,-1) WinPop

For(ii=0;ii<NumTraces;ii+=1)

	First=StringFromList(ii,TNL)
	TraceY=TraceYFromGraph(TWName,First)
	
	HashPos=StrSearch(First,"#",0)
	If(HashPos > 0)
		First=First[0,HashPos-1]		//remove "#n"
	Endif
	

	If(TraceY > 0)
		Duplicate /o/r=[][TraceY,TraceY] $First calc_Line
		WinPop[][ii]=calc_Line[p]
		WinPopOrigins[ii]=First+"[]["+Num2Str(TraceY)+"]"
	Else
		Duplicate /o $First calc_Line
		WinPop[][ii]=calc_Line[p]
		WinPopOrigins[ii]=First
	EndIf

EndFor


killwaves /z calc_Line
End

////////////////////////////////////////////////////////////////////

Function PopStats(PopWave)
Wave PopWave
Variable ii, numTraces, numPoints
numTraces=Dimsize(PopWave,1)
numPoints=DimSize(PopWave,0)

Duplicate /o PopWave, W_PopAvg, W_PopSD, W_PopSEM
Redimension /n=(-1) W_PopAvg, W_PopSD, W_PopSEM

W_PopAvg = 0
W_PopSD = 0

For(ii=0;ii<numTraces;ii+=1)

	W_PopAvg+=PopWave[p][ii]/numTraces

EndFor

For(ii=0;ii<numTraces;ii+=1)

	MultiThread W_PopSD[]+=(PopWave[p][ii]-W_PopAvg[p]) ^2
	

EndFor

MultiThread W_PopSD=sqrt(W_PopSD*(1/(NumTraces-1)))
MultiThread W_PopSEM=W_PopSD/Sqrt(NumTraces)


End

////////////////////////////////////////////////////////////
//For calculating median and percentiles
Function SortPop(pop,[resultname])
	Wave pop
	string resultname
	
	if(paramisdefault(resultname))
		resultname=nameofwave(pop)+"_sort"
	endif
	
	variable npts = dimsize(pop,0), ntraces = dimsize(pop,1), ii, bl
	
	if(ntraces < 1)
		ntraces = 1
	endif
	
	Duplicate /o/free pop, trace, NorPop
	redimension /n=(-1) trace
	Make/free/o Histo

	
	for(ii=0;ii<ntraces;ii+=1)
		trace = pop[p][ii]
		sort trace,trace
		
		NorPop[][ii] =trace[p]
	
	endfor
	
	setscale /i x,0,100,"",NorPop
	Duplicate /o NorPop, $resultname
	
	
End


////////////////////////////////////////////////////////////////////

Function AverageWavesFromWindow()

PopWaveFromWindow()
Wave WinPop

PopStats(WinPop)
Wave W_PopAvg, W_PopSD

AppendtoGraph w_PopAvg
ModifyGraph lsize(W_PopAvg)=2,rgb(W_PopAvg)=(0,0,0)

Display W_PopAvg
ErrorBars W_PopAvg Y,wave=(W_PopSD,W_PopSD)


End